@@ -1,5 +1,5 @@
 /*
- * "$Id: print-olympus.c,v 1.61.2.3 2007/12/29 20:42:28 rlk Exp $"
+ * "$Id: print-olympus.c,v 1.95 2010/01/04 14:04:25 m0m Exp $"
  *
  *   Print plug-in DyeSub driver (formerly Olympus driver) for the GIMP.
  *
@@ -39,8 +39,6 @@
 #include <stdio.h>
 #include <limits.h>
 
-#define STP_DBG_DYESUB STP_DBG_OLYMPUS
-
 #ifdef __GNUC__
 #define inline __inline__
 #endif
@@ -52,6 +50,7 @@
 #define DYESUB_FEATURE_BORDERLESS	0x00000008
 #define DYESUB_FEATURE_WHITE_BORDER	0x00000010
 #define DYESUB_FEATURE_PLANE_INTERLACE	0x00000020
+#define DYESUB_FEATURE_PLANE_LEFTTORIGHT	0x00000040
 
 #define DYESUB_PORTRAIT	0
 #define DYESUB_LANDSCAPE	1
@@ -139,6 +138,8 @@ typedef struct {
   size_t n_items;
 } laminate_list_t;
 
+#define NPUTC_BUFSIZE (4096)
+
 typedef struct
 {
   int w_dpi, h_dpi;
@@ -149,6 +150,7 @@ typedef struct
   const char* pagesize;
   const laminate_t* laminate;
   int print_mode;
+  char nputc_buf[NPUTC_BUFSIZE];
 } dyesub_privdata_t;
 
 static dyesub_privdata_t privdata;
@@ -166,6 +168,8 @@ typedef struct {
   int imgh_px, imgw_px;
   int prnh_px, prnw_px, prnt_px, prnb_px, prnl_px, prnr_px;
   int print_mode;	/* portrait or landscape */
+  int image_rows;
+  int plane_lefttoright;
 } dyesub_print_vars_t;
 
 typedef struct /* printer specific parameters */
@@ -849,6 +853,37 @@ static const dyesub_printsize_t cp220_printsize[] =
 
 LIST(dyesub_printsize_list_t, cp220_printsize_list, dyesub_printsize_t, cp220_printsize);
 
+
+/* Canon SELPHY CP-520 */
+static void cp520_printer_init_func(stp_vars_t *v)
+{
+  char pg = (strcmp(privdata.pagesize, "Postcard") == 0 ? '\1' :
+		(strcmp(privdata.pagesize, "w253h337") == 0 ? '\2' :
+		(strcmp(privdata.pagesize, "w155h244") == 0 ? '\3' :
+		(strcmp(privdata.pagesize, "w283h566") == 0 ? '\4' : 
+		 '\1' ))));
+
+  stp_put16_be(0x4000, v);
+  stp_putc('\0', v);
+  stp_putc(pg, v);
+  dyesub_nputc(v, '\0', 8); 
+  /* The CP520 does not want the printer_init and plane_init command to be sent
+     in the same USB-packet so we fill up first USB-Packet  with '\0'. */
+  dyesub_nputc(v, '\0', 1012); 
+}
+
+static void cp520_plane_init_func(stp_vars_t *v)
+{
+  stp_put16_be(0x4001, v);
+  stp_putc(3 - privdata.plane, v);  /* The CP520 differs from the cp-printer
+                                       in that it reqires the plane in the 3rd
+                                       byte not in the 4th */
+  stp_putc('\0', v);
+  stp_put32_le(privdata.w_size * privdata.h_size, v);
+  dyesub_nputc(v, '\0', 4);
+}
+
+
 /* Canon SELPHY ES series */
 static void es1_printer_init_func(stp_vars_t *v)
 {
@@ -972,7 +1007,10 @@ static const dyesub_pagesize_t updp10_page[] =
 {
   { "w288h432", "UPC-10P23 (4x6)", -1, -1, 12, 12, 18, 18, DYESUB_LANDSCAPE},
   { "w288h387", "UPC-10P34 (4x5)", -1, 384, 12, 12, 16, 16, DYESUB_LANDSCAPE},
+#if 0
+  /* We can't have two paper sizes that are the same size --rlk 20080813 */
   { "w288h432", "UPC-10S01 (Sticker)", -1, -1, 12, 12, 18, 18, DYESUB_LANDSCAPE},
+#endif
   { "Custom", NULL, -1, -1, 12, 12, 0, 0, DYESUB_LANDSCAPE},
 };
 
@@ -1223,7 +1261,7 @@ static void updr150_printer_end_func(stp_vars_t *v)
 static const dyesub_pagesize_t cx400_page[] =
 {
   { "w288h432", NULL, 295, 428, 24, 24, 23, 22, DYESUB_PORTRAIT},
-  { "w288h387", "4x5 3/8 (Digital Camera 3:4)", 295, 386, 24, 24, 23, 23, DYESUB_PORTRAIT},
+  { "w288h387", "4x5 3/8", 295, 386, 24, 24, 23, 23, DYESUB_PORTRAIT},
   { "w288h504", NULL, 295, 513, 24, 24, 23, 22, DYESUB_PORTRAIT},
   { "Custom", NULL, 295, 428, 0, 0, 0, 0, DYESUB_PORTRAIT},
 };
@@ -1401,6 +1439,89 @@ static void shinko_chcs9045_printer_init(stp_vars_t *v)
 }
 
 
+/* Dai Nippon Printing DS40 */
+static const dyesub_pagesize_t dnpds40_dock_page[] =
+{
+  { "w288h432", "4x6", PT(1920,300)+1, PT(1240,300)+1, 0, 0, 0, 0,
+						DYESUB_PORTRAIT},
+  { "w432h576", "6x9", PT(1920,300)+1, PT(2740,300)+1, 0, 0, 0, 0,
+							DYESUB_PORTRAIT},
+  { "A5", "6x8", PT(1920,300)+1, PT(2436,300)+1, 0, 0, 0, 0,
+						DYESUB_PORTRAIT},
+};
+
+LIST(dyesub_pagesize_list_t, dnpds40_dock_page_list, dyesub_pagesize_t, dnpds40_dock_page);
+
+static const dyesub_printsize_t dnpds40_dock_printsize[] =
+{
+  { "300x300", "w288h432", 1920, 1240},
+  { "300x300", "w432h576", 1920, 2740},
+  { "300x300", "A5", 1920, 2436},
+};
+
+LIST(dyesub_printsize_list_t, dnpds40_dock_printsize_list, dyesub_printsize_t, dnpds40_dock_printsize);
+
+static void dnpds40_printer_end(stp_vars_t *v)
+{
+  stp_zprintf(v, "\033PCNTRL START"); dyesub_nputc(v, ' ', 19);
+}
+
+static void dnpds40_plane_init(stp_vars_t *v)
+{
+  char p = (privdata.plane == 3 ? 'Y' :
+	    (privdata.plane == 2 ? 'M' :
+	     'C' ));
+
+  long RFSize = (privdata.w_size*privdata.h_size) + 1024 + 54;
+  long AdSize = (32 - (RFSize % 32));
+  long FSize = RFSize + AdSize;
+
+  stp_zprintf(v, "\033PCNTRL RETENTION       0000000800000000");
+  stp_zprintf(v, "\033PIMAGE %cPLANE", p); dyesub_nputc(v, ' ', 10);
+
+  stp_zprintf(v, "0%ld", FSize);
+  stp_zprintf(v, "BM");
+  stp_put32_le(FSize, v);
+  dyesub_nputc(v, '\0', 4);
+  stp_put32_le(1088, v);
+  stp_put32_le(40, v);
+  stp_put32_le(privdata.w_size, v);
+  stp_put32_le(privdata.h_size, v);
+  stp_put16_le(1, v);
+  stp_put16_le(8, v);
+  dyesub_nputc(v, '\0', 24);
+  dyesub_nputc(v, '\0', 1024);   /*RGB Array*/
+  dyesub_nputc(v, '\0', AdSize); /*Locate to 32bit border */
+}
+
+
+
+/* Dai Nippon Printing DS80 */
+static const dyesub_pagesize_t dnpds80_dock_page[] =
+{
+  { "c8x10", "8x10", PT(2560,300)+1, PT(3036,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},
+  { "C6", "8x4", PT(2560,300)+1, PT(1236,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},/* 8x4 */
+  { "C5", "8x5", PT(2560,300)+1, PT(1536,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},
+  { "C4", "8x6", PT(2560,300)+1, PT(1836,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},
+  { "C3", "8x8", PT(2560,300)+1, PT(2436,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},
+  { "C2", "8x12", PT(2560,300)+1, PT(3636,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},
+  { "C1", "A4 Length", PT(2560,300)+1, PT(3544,300)+1, 0, 0, 0, 0, DYESUB_PORTRAIT},
+};
+
+LIST(dyesub_pagesize_list_t, dnpds80_dock_page_list, dyesub_pagesize_t, dnpds80_dock_page);
+
+static const dyesub_printsize_t dnpds80_dock_printsize[] =
+{
+  { "300x300", "c8x10", 2560, 3036},
+  { "300x300", "C6", 2560, 1236},
+  { "300x300", "C5", 2560, 1536},
+  { "300x300", "C4", 2560, 1836},
+  { "300x300", "C3", 2560, 2436},
+  { "300x300", "C2", 2560, 3636},
+  { "300x300", "C1", 2560, 3544},
+};
+
+LIST(dyesub_printsize_list_t, dnpds80_dock_printsize_list, dyesub_printsize_t, dnpds80_dock_printsize);
 
 /* Model capabilities */
 
@@ -1544,6 +1665,22 @@ static const dyesub_cap_t dyesub_model_capabilities[] =
     cpx00_adj_cyan, cpx00_adj_magenta, cpx00_adj_yellow,
     NULL,
   },
+  { /* Canon CP-520, SELPHY CP-530 */
+    1004,
+    &ymc_ink_list,
+    &res_300dpi_list,
+    &cp220_page_list,
+    &cp220_printsize_list,
+    SHRT_MAX,
+    DYESUB_FEATURE_FULL_WIDTH | DYESUB_FEATURE_FULL_HEIGHT
+      | DYESUB_FEATURE_BORDERLESS | DYESUB_FEATURE_WHITE_BORDER
+      | DYESUB_FEATURE_PLANE_INTERLACE,
+    &cp520_printer_init_func, NULL,
+    &cp520_plane_init_func, NULL,
+    NULL, NULL,
+    cpx00_adj_cyan, cpx00_adj_magenta, cpx00_adj_yellow,
+    NULL,
+  },
   { /* Canon SELPHY ES1, ES2, ES20 (!experimental) */
     1003,
     &ymc_ink_list,
@@ -1691,6 +1828,36 @@ static const dyesub_cap_t dyesub_model_capabilities[] =
     NULL, NULL, NULL,
     NULL,
   },
+  { /* Dai Nippon Printing DS40 */
+    6000,
+    &rgb_ink_list,
+    &res_300dpi_list,
+    &dnpds40_dock_page_list,
+    &dnpds40_dock_printsize_list,
+    SHRT_MAX,
+    DYESUB_FEATURE_FULL_WIDTH | DYESUB_FEATURE_FULL_HEIGHT
+      | DYESUB_FEATURE_PLANE_INTERLACE | DYESUB_FEATURE_PLANE_LEFTTORIGHT,
+    NULL, &dnpds40_printer_end,
+    &dnpds40_plane_init, NULL,
+    NULL, NULL,
+    NULL, NULL, NULL,
+    NULL,
+  },
+  { /* Dai Nippon Printing DS80 */
+    6001,
+    &rgb_ink_list,
+    &res_300dpi_list,
+    &dnpds80_dock_page_list,
+    &dnpds80_dock_printsize_list,
+    SHRT_MAX,
+    DYESUB_FEATURE_FULL_WIDTH | DYESUB_FEATURE_FULL_HEIGHT
+      | DYESUB_FEATURE_PLANE_INTERLACE | DYESUB_FEATURE_PLANE_LEFTTORIGHT,
+    NULL, &dnpds40_printer_end,
+    &dnpds40_plane_init, NULL,
+    NULL, NULL,
+    NULL, NULL, NULL,
+    NULL,
+  },
 };
 
 static const stp_parameter_t the_parameters[] =
@@ -1699,49 +1866,53 @@ static const stp_parameter_t the_parameters[] =
     "PageSize", N_("Page Size"), N_("Basic Printer Setup"),
     N_("Size of the paper being printed to"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "MediaType", N_("Media Type"), N_("Basic Printer Setup"),
     N_("Type of media (plain paper, photo paper, etc.)"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "InputSlot", N_("Media Source"), N_("Basic Printer Setup"),
     N_("Source (input slot) of the media"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "Resolution", N_("Resolution"), N_("Basic Printer Setup"),
     N_("Resolution and quality of the print"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "InkType", N_("Ink Type"), N_("Advanced Printer Setup"),
     N_("Type of ink in the printer"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
+    /* TRANSLATORS: Some dye sublimation printers are able achieve */
+    /* better durability of output by covering it with transparent */
+    /* laminate surface. This surface can be of different patterns: */
+    /* common are matte, glossy or texture. */
     "Laminate", N_("Laminate Pattern"), N_("Advanced Printer Setup"),
     N_("Laminate Pattern"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 0, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 0, STP_CHANNEL_NONE, 1, 0
   },
   {
     "Borderless", N_("Borderless"), N_("Advanced Printer Setup"),
     N_("Print without borders"),
     STP_PARAMETER_TYPE_BOOLEAN, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 0, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 0, STP_CHANNEL_NONE, 1, 0
   },
   {
     "PrintingMode", N_("Printing Mode"), N_("Core Parameter"),
     N_("Printing Output Mode"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -2169,9 +2340,24 @@ dyesub_describe_output(const stp_vars_t *v)
 static void
 dyesub_nputc(stp_vars_t *v, char byte, int count)
 {
-  int i;
-  for (i = 0; i < count; i++)
+  if (count == 1)
     stp_putc(byte, v);
+  else
+    {
+      int i;
+      char *buf = privdata.nputc_buf;
+      int size = count;
+      int blocks = size / NPUTC_BUFSIZE;
+      int leftover = size % NPUTC_BUFSIZE;
+      if (size > NPUTC_BUFSIZE)
+	size = NPUTC_BUFSIZE;
+      (void) memset(buf, byte, size);
+      if (blocks)
+	for (i = 0; i < blocks; i++)
+	  stp_zfwrite(buf, size, 1, v);
+      if (leftover)
+	stp_zfwrite(buf, leftover, 1, v);
+    }
 }
 
 static void
@@ -2222,9 +2408,10 @@ dyesub_interpolate(int oldval, int oldsize, int newsize)
 }
 
 static void
-dyesub_free_image(unsigned short** image_data, stp_image_t *image)
+dyesub_free_image(dyesub_print_vars_t *pv, stp_image_t *image)
 {
-  int image_px_height = stp_image_height(image);
+  unsigned short** image_data = pv->image_data;
+  int image_px_height = pv->image_rows;
   int i;
 
   for (i = 0; i< image_px_height; i++)
@@ -2247,6 +2434,7 @@ dyesub_read_image(stp_vars_t *v,
   int i;
 
   image_data = stp_zalloc(image_px_height * sizeof(unsigned short *));
+  pv->image_rows = 0;
   if (!image_data)
     return NULL;	/* ? out of memory ? */
 
@@ -2257,22 +2445,21 @@ dyesub_read_image(stp_vars_t *v,
 	  stp_deprintf(STP_DBG_DYESUB,
 	  	"dyesub_read_image: "
 		"stp_color_get_row(..., %d, ...) == 0\n", i);
-	  dyesub_free_image(image_data, image);
+	  dyesub_free_image(pv, image);
 	  return NULL;
 	}	
       image_data[i] = stp_malloc(row_size);
+      pv->image_rows = i+1;
       if (!image_data[i])
         {
 	  stp_deprintf(STP_DBG_DYESUB,
 	  	"dyesub_read_image: "
 		"(image_data[%d] = stp_malloc()) == NULL\n", i);
-	  dyesub_free_image(image_data, image);
+	  dyesub_free_image(pv, image);
 	  return NULL;
 	}	
       memcpy(image_data[i], stp_channel_get_output(v), row_size);
     }
-  stp_image_conclude(image);
-
   return image_data;
 }
 
@@ -2344,7 +2531,10 @@ dyesub_print_row(stp_vars_t *v,
   for (w = 0; w < pv->outw_px; w++)
     {
       col = dyesub_interpolate(w, pv->outw_px, pv->imgw_px);
-      ret = dyesub_print_pixel(v, pv, row, col, plane);
+      if (pv->plane_lefttoright)
+	ret = dyesub_print_pixel(v, pv, row, pv->imgw_px - col - 1, plane);
+      else
+	ret = dyesub_print_pixel(v, pv, row, col, plane);
       if (ret > 1)
       	break;
     }
@@ -2451,6 +2641,7 @@ dyesub_do_print(stp_vars_t *v, stp_image_t *image)
       stp_eprintf(v, _("Print options not verified; cannot print.\n"));
       return 0;
     }
+  (void) memset(&pv, 0, sizeof(pv));
 
   stp_image_init(image);
   pv.imgw_px = stp_image_width(image);
@@ -2531,9 +2722,13 @@ dyesub_do_print(stp_vars_t *v, stp_image_t *image)
  		(strcmp(ink_type, "RGB") == 0 || strcmp(ink_type, "BGR") == 0)
 		? '\xff' : '\0');
   pv.plane_interlacing = dyesub_feature(caps, DYESUB_FEATURE_PLANE_INTERLACE);
+  pv.plane_lefttoright = dyesub_feature(caps, DYESUB_FEATURE_PLANE_LEFTTORIGHT);
   pv.print_mode = page_mode;
   if (!pv.image_data)
-      return 2;	
+    {
+      stp_image_conclude(image);
+      return 2;
+    }
   /* /FIXME */
 
 
@@ -2542,7 +2737,6 @@ dyesub_do_print(stp_vars_t *v, stp_image_t *image)
   dyesub_adjust_curve(v, caps->adj_yellow, "YellowCurve");
   stp_set_float_parameter(v, "Density", 1.0);
 
-
   if (dyesub_feature(caps, DYESUB_FEATURE_FULL_HEIGHT))
     {
       pv.prnt_px = 0;
@@ -2613,7 +2807,8 @@ dyesub_do_print(stp_vars_t *v, stp_image_t *image)
   /* printer end */
   dyesub_exec(v, caps->printer_end_func, "caps->printer_end");
 
-  dyesub_free_image(pv.image_data, image);
+  dyesub_free_image(&pv, image);
+  stp_image_conclude(image);
   return status;
 }
 
@@ -2670,16 +2865,16 @@ print_dyesub_module_exit(void)
 
 
 /* Module header */
-#define stp_module_version print_olympus_LTX_stp_module_version
-#define stp_module_data print_olympus_LTX_stp_module_data
+#define stp_module_version print_dyesub_LTX_stp_module_version
+#define stp_module_data print_dyesub_LTX_stp_module_data
 
 stp_module_version_t stp_module_version = {0, 0};
 
 stp_module_t stp_module_data =
   {
-    "olympus",
+    "dyesub",
     VERSION,
-    "Olympus family driver",
+    "DyeSub family driver",
     STP_MODULE_CLASS_FAMILY,
     NULL,
     print_dyesub_module_init,
