@@ -1,5 +1,5 @@
 /*
- * "$Id: printers.c,v 1.79.2.2 2007/12/15 20:35:50 rlk Exp $"
+ * "$Id: printers.c,v 1.86 2009/10/05 12:37:30 rlk Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -65,7 +65,9 @@ struct stp_printer
   char       *long_name;        /* Long name for UI */
   char       *family;           /* Printer family */
   char	     *manufacturer;	/* Printer manufacturer */
+  char	     *device_id; /* IEEE 1284 device ID */
   int        model;             /* Model number */
+  int	     vars_initialized;
   const stp_printfuncs_t *printfuncs;
   stp_vars_t *printvars;
 };
@@ -185,6 +187,12 @@ stpi_printer_long_namefunc(const void *item)
   return printer->long_name;
 }
 
+const char *
+stp_printer_get_device_id(const stp_printer_t *printer)
+{
+  return printer->device_id;
+}
+
 const char *
 stp_printer_get_family(const stp_printer_t *printer)
 {
@@ -209,12 +217,6 @@ stpi_get_printfuncs(const stp_printer_t *printer)
   return printer->printfuncs;
 }
 
-const stp_vars_t *
-stp_printer_get_defaults(const stp_printer_t *printer)
-{
-  return printer->printvars;
-}
-
 
 const stp_printer_t *
 stp_get_printer_by_long_name(const char *long_name)
@@ -248,6 +250,30 @@ stp_get_printer_by_driver(const char *driver)
   return (const stp_printer_t *) stp_list_item_get_data(printer_item);
 }
 
+const stp_printer_t *
+stp_get_printer_by_device_id(const char *device_id)
+{
+  stp_list_item_t *printer_item;
+  if (printer_list == NULL)
+    {
+      stp_erprintf("No printer drivers found: "
+		   "are STP_DATA_PATH and STP_MODULE_PATH correct?\n");
+      stpi_init_printer_list();
+    }
+  if (! device_id || strcmp(device_id, "") == 0)
+    return NULL;
+
+  printer_item = stp_list_get_start(printer_list);
+  while (printer_item)
+    {
+      if (strcmp(((const stp_printer_t *) stp_list_item_get_data(printer_item)),
+		 device_id) == 0)
+	return ((const stp_printer_t *) stp_list_item_get_data(printer_item));
+      printer_item = stp_list_item_next(printer_item);
+    }
+  return NULL;
+}
+
 int
 stp_get_printer_index_by_driver(const char *driver)
 {
@@ -293,7 +319,7 @@ stp_printer_describe_parameter(const stp_vars_t *v, const char *name,
 }
 
 static void
-set_printer_defaults(stp_vars_t *v, int core_only)
+set_printer_defaults(stp_vars_t *v, int core_only, int soft)
 {
   stp_parameter_list_t *params;
   int count;
@@ -311,32 +337,60 @@ set_printer_defaults(stp_vars_t *v, int core_only)
 	  switch (p->p_type)
 	    {
 	    case STP_PARAMETER_TYPE_STRING_LIST:
-	      stp_set_string_parameter(v, p->name, desc.deflt.str);
-	      stp_set_string_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_string_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_string_parameter(v, p->name, desc.deflt.str);
+		  stp_set_string_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    case STP_PARAMETER_TYPE_DOUBLE:
-	      stp_set_float_parameter(v, p->name, desc.deflt.dbl);
-	      stp_set_float_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_float_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_float_parameter(v, p->name, desc.deflt.dbl);
+		  stp_set_float_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    case STP_PARAMETER_TYPE_INT:
-	      stp_set_int_parameter(v, p->name, desc.deflt.integer);
-	      stp_set_int_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_int_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_int_parameter(v, p->name, desc.deflt.integer);
+		  stp_set_int_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    case STP_PARAMETER_TYPE_DIMENSION:
-	      stp_set_dimension_parameter(v, p->name, desc.deflt.dimension);
-	      stp_set_dimension_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_dimension_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_dimension_parameter(v, p->name, desc.deflt.dimension);
+		  stp_set_dimension_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    case STP_PARAMETER_TYPE_BOOLEAN:
-	      stp_set_boolean_parameter(v, p->name, desc.deflt.boolean);
-	      stp_set_boolean_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_boolean_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_boolean_parameter(v, p->name, desc.deflt.boolean);
+		  stp_set_boolean_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    case STP_PARAMETER_TYPE_CURVE:
-	      stp_set_curve_parameter(v, p->name, desc.deflt.curve);
-	      stp_set_curve_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_curve_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_curve_parameter(v, p->name, desc.deflt.curve);
+		  stp_set_curve_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    case STP_PARAMETER_TYPE_ARRAY:
-	      stp_set_array_parameter(v, p->name, desc.deflt.array);
-	      stp_set_array_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+	      if (!soft ||
+		  !stp_check_array_parameter(v, p->name, STP_PARAMETER_DEFAULTED))
+		{
+		  stp_set_array_parameter(v, p->name, desc.deflt.array);
+		  stp_set_array_parameter_active(v, p->name, STP_PARAMETER_ACTIVE);
+		}
 	      break;
 	    default:
 	      break;
@@ -351,13 +405,19 @@ void
 stp_set_printer_defaults(stp_vars_t *v, const stp_printer_t *printer)
 {
   stp_set_driver(v, stp_printer_get_driver(printer));
-  set_printer_defaults(v, 0);
+  set_printer_defaults(v, 0, 0);
+}
+
+void
+stp_set_printer_defaults_soft(stp_vars_t *v, const stp_printer_t *printer)
+{
+  stp_set_driver(v, stp_printer_get_driver(printer));
+  set_printer_defaults(v, 0, 1);
 }
 
 void
 stp_initialize_printer_defaults(void)
 {
-  stp_list_item_t *printer_item;
   if (printer_list == NULL)
     {
       stpi_init_printer_list();
@@ -365,16 +425,19 @@ stp_initialize_printer_defaults(void)
 	(STP_DBG_PRINTERS,
 	 "stpi_family_register(): initialising printer_list...\n");
     }
-  printer_item = stp_list_get_start(printer_list);
-  while (printer_item)
+}
+
+const stp_vars_t *
+stp_printer_get_defaults(const stp_printer_t *printer)
+{
+  if (! printer->vars_initialized)
     {
-      stp_deprintf
-	(STP_DBG_PRINTERS, "  ==>init %s\n",
-	 ((stp_printer_t *)(stp_list_item_get_data(printer_item)))->driver);
-      set_printer_defaults
-	(((stp_printer_t *)(stp_list_item_get_data(printer_item)))->printvars, 1);
-      printer_item = stp_list_item_next(printer_item);
+      stp_printer_t *nc_printer = (stp_printer_t *) printer;
+      stp_deprintf(STP_DBG_PRINTERS, "  ==>init %s\n", printer->driver);
+      set_printer_defaults (nc_printer->printvars, 1, 0);
+      nc_printer->vars_initialized = 1;
     }
+  return printer->printvars;
 }
 
 void
@@ -738,10 +801,6 @@ stp_verify_printer_params(stp_vars_t *v)
   errbuf_t errbuf;
   stp_outfunc_t ofunc = stp_get_errfunc(v);
   void *odata = stp_get_errdata(v);
-
-  stp_dprintf(STP_DBG_VARS, v, "** Entering stp_verify_printer_params(0x%p)\n",
-	      v);
-
   stp_parameter_list_t params;
   int nparams;
   int i;
@@ -749,6 +808,9 @@ stp_verify_printer_params(stp_vars_t *v)
   int left, top, bottom, right;
   const char *pagesize = stp_get_string_parameter(v, "PageSize");
 
+  stp_dprintf(STP_DBG_VARS, v, "** Entering stp_verify_printer_params(0x%p)\n",
+	      (void *) v);
+
   stp_set_errfunc((stp_vars_t *) v, fill_buffer_writefunc);
   stp_set_errdata((stp_vars_t *) v, &errbuf);
 
@@ -848,7 +910,7 @@ stp_verify_printer_params(stp_vars_t *v)
       stp_free(errbuf.data);
     }
   stp_dprintf(STP_DBG_VARS, v, "** Exiting stp_verify_printer_params(0x%p) => %d\n",
-	      v, answer);
+	      (void *) v, answer);
   return answer;
 }
 
@@ -901,8 +963,7 @@ stp_family_register(stp_list_t *family)
       while(printer_item)
 	{
 	  printer = (const stp_printer_t *) stp_list_item_get_data(printer_item);
-	  if (!stp_list_get_item_by_name(printer_list,
-					  stp_get_driver(printer->printvars)))
+	  if (!stp_list_get_item_by_name(printer_list, printer->driver))
 	    stp_list_item_create(printer_list, NULL, printer);
 	  printer_item = stp_list_item_next(printer_item);
 	}
@@ -934,8 +995,7 @@ stp_family_unregister(stp_list_t *family)
 	{
 	  printer = (const stp_printer_t *) stp_list_item_get_data(printer_item);
 	  old_printer_item =
-	    stp_list_get_item_by_name(printer_list,
-				      stp_get_driver(printer->printvars));
+	    stp_list_get_item_by_name(printer_list, printer->driver);
 
 	  if (old_printer_item)
 	    stp_list_item_destroy(printer_list, old_printer_item);
@@ -945,105 +1005,6 @@ stp_family_unregister(stp_list_t *family)
   return 0;
 }
 
-static void
-stp_fill_printvars_from_xmltree(stp_mxml_node_t *prop,
-				stp_vars_t *v)
-{
-  while (prop)
-    {
-      if (prop->type == STP_MXML_ELEMENT)
-	{
-	  const char *prop_name = prop->value.element.name;
-	  if (!strcmp(prop_name, "parameter"))
-	    {
-	      const char *p_type = stp_mxmlElementGetAttr(prop, "type");
-	      const char *p_name = stp_mxmlElementGetAttr(prop, "name");
-	      stp_mxml_node_t *child = prop->child;
-	      if (!p_type || !p_name)
-		stp_erprintf("Bad property found!\n");
-	      else if (strcmp(p_type, "float") == 0)
-		{
-		  if (child->type == STP_MXML_TEXT)
-		    {
-		      stp_set_float_parameter
-			(v, p_name, stp_xmlstrtod(child->value.text.string));
-		      stp_deprintf(STP_DBG_XML, "  Set float '%s' to '%s' (%f)\n",
-				   p_name, child->value.text.string,
-				   stp_get_float_parameter(v, p_name));
-		    }
-		}
-	      else if (strcmp(p_type, "integer") == 0)
-		{
-		  if (child->type == STP_MXML_TEXT)
-		    {
-		      stp_set_int_parameter
-			(v, p_name, (int) stp_xmlstrtol(child->value.text.string));
-		      stp_deprintf(STP_DBG_XML, "  Set int '%s' to '%s' (%d)\n",
-				   p_name, child->value.text.string,
-				   stp_get_int_parameter(v, p_name));
-		    }
-		}
-	      else if (strcmp(p_type, "boolean") == 0)
-		{
-		  if (child->type == STP_MXML_TEXT)
-		    {
-		      stp_set_boolean_parameter
-			(v, p_name, (int) stp_xmlstrtol(child->value.text.string));
-		      stp_deprintf(STP_DBG_XML, "  Set bool '%s' to '%s' (%d)\n",
-				   p_name, child->value.text.string,
-				   stp_get_boolean_parameter(v, p_name));
-		    }
-		}
-	      else if (strcmp(p_type, "string") == 0)
-		{
-		  if (child->type == STP_MXML_TEXT)
-		    {
-		      stp_set_string_parameter
-			(v, p_name, child->value.text.string);
-		      stp_deprintf(STP_DBG_XML, "  Set string '%s' to '%s' (%s)\n",
-				   p_name, child->value.text.string,
-				   stp_get_string_parameter(v, p_name));
-		    }
-		}
-	      else if (strcmp(p_type, "curve") == 0)
-		{
-		  stp_curve_t *curve;
-		  while (child->type == STP_MXML_TEXT && child->next)
-		    child = child->next;
-		  curve = stp_curve_create_from_xmltree(child);
-		  if (curve)
-		    {
-		      stp_set_curve_parameter(v, p_name, curve);
-		      stp_deprintf(STP_DBG_XML, "  Set curve '%s' to '%s' (%s)\n",
-				   p_name, child->value.text.string,
-				   stp_curve_write_string(curve));
-		      stp_curve_destroy(curve);
-		    }
-		}
-	      else if (strcmp(p_type, "array") == 0)
-		{
-		  stp_array_t *array;
-		  while (child->type == STP_MXML_TEXT && child->next)
-		    child = child->next;
-		  array = stp_array_create_from_xmltree(child);
-		  if (array)
-		    {
-		      stp_set_array_parameter(v, p_name, array);
-		      stp_deprintf(STP_DBG_XML, "  Set array '%s' to '%s'\n",
-				   p_name, child->value.text.string);
-		      stp_array_destroy(array);
-		    }
-		}
-	      else
-		{
-		  stp_erprintf("Bad property %s type %s\n", p_name, p_type);
-		}
-	    }
-	}
-      prop = prop->next;
-    }
-} 
-
 static stp_printvars_t *
 stp_printvars_create_from_xmltree(stp_mxml_node_t *printer,
 				  const char *family)
@@ -1075,10 +1036,10 @@ stp_printvars_create_from_xmltree(stp_mxml_node_t *printer,
   outprintvars->name = sbuf;
   prop = printer->child;
   stp_deprintf(STP_DBG_XML, ">>stp_printvars_create_from_xmltree: %p, %s\n",
-	       outprintvars->printvars, outprintvars->name);
-  stp_fill_printvars_from_xmltree(prop, outprintvars->printvars);
+	       (void *) (outprintvars->printvars), outprintvars->name);
+  stp_vars_fill_from_xmltree(prop, outprintvars->printvars);
   stp_deprintf(STP_DBG_XML, "<<stp_printvars_create_from_xmltree: %p, %s\n",
-	       outprintvars->printvars, outprintvars->name);
+	       (void *) (outprintvars->printvars), outprintvars->name);
   return outprintvars;
 }
 
@@ -1124,6 +1085,9 @@ stp_printer_create_from_xmltree(stp_mxml_node_t *printer, /* The printer node */
   outprinter->manufacturer = stp_strdup(stp_mxmlElementGetAttr(printer, "manufacturer"));
   outprinter->model = stp_xmlstrtol(stp_mxmlElementGetAttr(printer, "model"));
   outprinter->family = stp_strdup((const char *) family);
+  stmp = stp_mxmlElementGetAttr(printer, "deviceid");
+  if (stmp)
+    outprinter->device_id = stp_strdup(stmp);
 
   if (stp_get_driver(outprinter->printvars))
     driver = 1;
@@ -1133,7 +1097,7 @@ stp_printer_create_from_xmltree(stp_mxml_node_t *printer, /* The printer node */
   outprinter->printfuncs = printfuncs;
 
   prop = printer->child;
-  stp_fill_printvars_from_xmltree(prop, outprinter->printvars);
+  stp_vars_fill_from_xmltree(prop, outprinter->printvars);
   if (driver && long_name && printfuncs)
     {
       if (stp_get_debug_level() & STP_DBG_XML)
